@@ -41,7 +41,7 @@ class EndpointParseResponse(enum.Enum):
     ENUM_TYPE_REQUIRES_VALUES_CONSTRAINT = "enum_type_requires_values_constraint"
     CONSTRAINT_VALUES_MUST_BE_DICT = "constraint_values_must_be_dict"
     CONSTRAINT_VALUES_KEYS_MUST_BE_STRINGS = "constraint_values_keys_must_be_strings"
-    CONSTRAINT_VALUES_VALUES_MUST_BE_STRINGS = "constraint_values_values_must_be_strings"
+    CONSTRAINT_VALUES_VALUES_MUST_BE_INT_OR_STRING = "constraint_values_values_must_be_int_or_string"
 
 
 class EndpointType(enum.Enum):
@@ -62,13 +62,12 @@ class AbstractEndpointValidator:
     def validate(self, data) -> ValidationResponse:
         pass
 
-    def __dict__(self):
-        return {
-            "id": self.id,
-            "name": self.name,
-            "type": self.type,
-            "specialId": self.special_id
-        }
+    def __iter__(self):
+        yield "id", self.id
+        yield "name", self.name
+        yield "type", self.type
+        yield "specialId", self.special_id
+        yield "constraints", None
 
     @staticmethod
     def parse(endpoint_data: dict) -> Union['AbstractEndpointValidator', EndpointParseResponse]:
@@ -99,21 +98,22 @@ class BooleanEndpointValidator(AbstractEndpointValidator):
         super().__init__(endpoint_id, name, special_id)
         self.type = EndpointType.BOOLEAN
 
-    def validate(self, data) -> ValidationResponse:
+    def validate(self, data) -> Tuple[ValidationResponse, Union[bool, None]]:
         # check that data is a boolean value
         if not isinstance(data, bool):
-            return ValidationResponse.INVALID_TYPE
+            try:
+                data = bool(data)
+            except ValueError:
+                return ValidationResponse.INVALID_TYPE, None
 
-        return ValidationResponse.VALID
+        return ValidationResponse.VALID, data
 
-    def __dict__(self):
-        return {
-            "id": self.id,
-            "name": self.name,
-            "type": self.type.value,
-            "specialId": self.special_id,
-            "constraints": None
-        }
+    def __iter__(self):
+        yield "id", self.id
+        yield "name", self.name
+        yield "type", self.type
+        yield "specialId", self.special_id
+        yield "constraints", None
 
     @staticmethod
     def parse(endpoint_data: dict) -> Union['BooleanEndpointValidator', EndpointParseResponse]:
@@ -141,36 +141,37 @@ class IntegerEndpointValidator(AbstractEndpointValidator):
         self.max = max_v
         self.increment = increment
 
-    def validate(self, data) -> ValidationResponse:
+    def validate(self, data) -> Tuple[ValidationResponse, Union[int, None]]:
         # check if it is a number
         if not isinstance(data, int):
-            return ValidationResponse.INVALID_TYPE
+            try:
+                data = int(data)
+            except ValueError:
+                return ValidationResponse.INVALID_TYPE, None
 
         # check if a minimum value was provided
         if self.min is not None:
             # check if it is less than the minimum
             if data < self.min:
-                return ValidationResponse.LOWER_THAN_MIN
+                return ValidationResponse.LOWER_THAN_MIN, None
 
         # check if a maximum value was provided
         if self.max is not None:
             # check if it is more than the maximum
             if data > self.max:
-                return ValidationResponse.HIGHER_THAN_MAX
+                return ValidationResponse.HIGHER_THAN_MAX, None
 
-        return ValidationResponse.VALID
+        return ValidationResponse.VALID, data
 
-    def __dict__(self):
-        return {
-            "id": self.id,
-            "name": self.name,
-            "type": self.type.value,
-            "specialId": self.special_id,
-            "constraints": {
-                "min": self.min,
-                "max": self.max,
-                "increment": self.increment
-            }
+    def __iter__(self):
+        yield "id", self.id
+        yield "name", self.name
+        yield "type", self.type
+        yield "specialId", self.special_id
+        yield "constraints", {
+            "min": self.min,
+            "max": self.max,
+            "increment": self.increment
         }
 
     @staticmethod
@@ -221,16 +222,19 @@ class StringEndpointValidator(AbstractEndpointValidator):
         self.allow_list = allow_list
         self.deny_list = deny_list
 
-    def validate(self, data) -> ValidationResponse:
+    def validate(self, data) -> Tuple[ValidationResponse, Union[str, None]]:
         # check if it is a string
         if not isinstance(data, str):
-            return ValidationResponse.INVALID_TYPE
+            try:
+                data = str(data)
+            except ValueError:
+                return ValidationResponse.INVALID_TYPE, None
 
         # check if the length constraint was provided
         if self.length is not None:
             # check if the length is correct
             if len(data) != self.length:
-                return ValidationResponse.INVALID_LENGTH
+                return ValidationResponse.INVALID_LENGTH, None
 
         # check if allow_list is provided
         if self.allow_list is not None:
@@ -239,12 +243,12 @@ class StringEndpointValidator(AbstractEndpointValidator):
                 for char in data:
                     # check if it is not in the allow_list
                     if char not in self.allow_list:
-                        return ValidationResponse.VIOLATES_ALLOW_LIST
+                        return ValidationResponse.VIOLATES_ALLOW_LIST, None
             # if the allow_list is an array of strings
             else:
                 # if allow_list is a list of strings check if provided string matches one of those strings
                 if data not in self.allow_list:
-                    return ValidationResponse.VIOLATES_ALLOW_LIST
+                    return ValidationResponse.VIOLATES_ALLOW_LIST, None
 
         # check if deny_list is provided
         if self.deny_list is not None:
@@ -253,24 +257,22 @@ class StringEndpointValidator(AbstractEndpointValidator):
                 for char in self.deny_list:
                     # check if the deny_list contains that char
                     if char in data:
-                        return ValidationResponse.VIOLATES_DENY_LIST
+                        return ValidationResponse.VIOLATES_DENY_LIST, None
             else:
                 if data in self.deny_list:
-                    return ValidationResponse.VIOLATES_DENY_LIST
+                    return ValidationResponse.VIOLATES_DENY_LIST, None
 
-        return ValidationResponse.VALID
+        return ValidationResponse.VALID, data
 
-    def __dict__(self):
-        return {
-            "id": self.id,
-            "name": self.name,
-            "type": self.type.value,
-            "specialId": self.special_id,
-            "constraints": {
-                "length": self.length,
-                "allowList": self.allow_list,
-                "denyList": self.deny_list
-            }
+    def __iter__(self):
+        yield "id", self.id
+        yield "name", self.name
+        yield "type", self.type
+        yield "specialId", self.special_id
+        yield "constraints", {
+            "length": self.length,
+            "allowList": self.allow_list,
+            "denyList": self.deny_list
         }
 
     @staticmethod
@@ -335,31 +337,38 @@ class StringEndpointValidator(AbstractEndpointValidator):
 
 class EnumEndpointValidator(AbstractEndpointValidator):
     def __init__(self, endpoint_id: str, name: str, special_id: Union[str, None],
-                 values: Dict[str, str]):
+                 values: Dict[str, Union[int, str]]):
         super().__init__(endpoint_id, name, special_id)
         self.type = EndpointType.ENUM
         self.values = values
 
-    def validate(self, data) -> ValidationResponse:
+    def validate(self, data) -> Tuple[ValidationResponse, Union[str, int, None]]:
         # check if it is a string
-        if not isinstance(data, str):
-            return ValidationResponse.INVALID_TYPE
+        if not isinstance(data, str) and not isinstance(data, int):
+            try:
+                data = str(data)
+            except ValueError:
+                return ValidationResponse.INVALID_TYPE, None
 
         # check if the value is a valid enum for this endpoint
-        if data not in self.values.keys():
-            return ValidationResponse.INVALID_ENUM_VALUE
+        if data not in self.values.values():
+            try:
+                data = int(data)
 
-        return ValidationResponse.VALID
+                if data not in self.values.values():
+                    return ValidationResponse.INVALID_ENUM_VALUE, None
+            except ValueError:
+                return ValidationResponse.INVALID_ENUM_VALUE, None
 
-    def __dict__(self):
-        return {
-            "id": self.id,
-            "name": self.name,
-            "type": self.type.value,
-            "specialId": self.special_id,
-            "constraints": {
-                "values": self.values
-            }
+        return ValidationResponse.VALID, data
+
+    def __iter__(self):
+        yield "id", self.id
+        yield "name", self.name
+        yield "type", self.type
+        yield "specialId", self.special_id
+        yield "constraints", {
+            "values": self.values
         }
 
     @staticmethod
@@ -389,8 +398,8 @@ class EnumEndpointValidator(AbstractEndpointValidator):
             for key, val in constraints.get("values").items():
                 if not isinstance(key, str):
                     return EndpointParseResponse.CONSTRAINT_VALUES_KEYS_MUST_BE_STRINGS
-                elif not isinstance(val, str):
-                    return EndpointParseResponse.CONSTRAINT_VALUES_VALUES_MUST_BE_STRINGS
+                elif not isinstance(val, str) and not isinstance(val, int):
+                    return EndpointParseResponse.CONSTRAINT_VALUES_VALUES_MUST_BE_INT_OR_STRING
 
         # return the validator
         return EnumEndpointValidator(
@@ -410,6 +419,10 @@ class EndpointManager:
     def initialized(self):
         return self.__initialized
 
+    @property
+    def endpoints(self):
+        return [dict(x) for x in self.__endpoints.values()]
+
     def __add_endpoint(self, validator: AbstractEndpointValidator):
         self.__endpoints[validator.id] = validator
 
