@@ -31,7 +31,7 @@ from .__main__ import __protocol_version__
 
 
 class IoTManagerMiddleware(object):
-    def __init__(self, app, wsgi_app, manager: 'IoTManager'):
+    def __init__(self, app: flask.Flask, wsgi_app, manager: 'IoTManager'):
         """
         Manager middleware for WSGI.
 
@@ -51,8 +51,9 @@ class IoTManagerMiddleware(object):
         if environ["PATH_INFO"].startswith("/iot.io"):
             # try to handle the request normally
             try:
-                # handle using websocket function
-                return self.ws(environ, start_response)
+                with self.app.request_context(environ):
+                    # handle using websocket function
+                    return self.ws(environ, start_response)
             # if an exception in the handler function is raised deal with it here
             except ConnectionFailed:
                 # start response for a 400 code
@@ -150,9 +151,6 @@ class IoTManager(object):
 
     # function for handling new websocket clients
     def socket(self, ws: WebSocket):
-        # parse out client information from url
-        args = parse(environ=ws.environ)
-
         # get headers
         headers = dict(ws.environ["headers_raw"])
 
@@ -220,7 +218,8 @@ class IoTManager(object):
         # check if the client has a valid type
         if self.__types.get(headers["IoT-IO-Type"], None) is None:
             # log and exit if the client has an invalid type
-            self.logger.warning("Client with ID '" + args["id"] + "' claimed a device type of '" + args["type"]
+            self.logger.warning("Client with ID '" + headers["IoT-IO-Id"] + "' claimed a device type of '"
+                                + headers["IoT-IO-Type"]
                                 + "' but no such device type was defined. Refusing connection.")
 
             # send error message to client
@@ -247,6 +246,12 @@ class IoTManager(object):
 
             raise ClientInvalidEndpoints()
 
+        # set the request info so we know which client this request belongs to
+        flask.request.g = {
+            "client": client
+        }
+
+        # add the client
         self.add(client)
 
         # client loop
@@ -456,7 +461,8 @@ class IoTManager(object):
         self.__types[device.type] = device
         self.logger.debug("Successfully added DeviceType '" + device.type + "'.")
 
-    def emit(self, event: str, data: sendable, client_id: str = None, client_type: str = None, room: str = None):
+    def emit(self, event: str, data: sendable, client_id: str = None, client_type: str = None, room: str = None,
+             trigger_update: bool = True):
         """
         Emit function for sending data to a single client, group of client types, or room of clients.
 
@@ -469,13 +475,16 @@ class IoTManager(object):
                             specifier.
         :param room: A room id of which all clients should be specified. If client_type is provided it will act as a
                      filter and only send the data ot clients of that type which are also in the specified room.
+        :param trigger_update: If this emit should trigger the managers on_update handler. This is useful for preventing
+                               unnecessary triggers of the on_update function if for example it is used to update a
+                               database for data persistence.
         :return:
         """
         if client_id is not None:
             client = self.__clients.get(client_id, None)
 
             if client:
-                client.emit(event, data)
+                client.emit(event, data, trigger_update)
         elif room is not None:
             if self.__rooms.get(room, None) is not None:
                 for client_id in self.__rooms[room]:
@@ -483,14 +492,14 @@ class IoTManager(object):
 
                     if client:
                         if client_type is None:
-                            client.emit(event, data)
+                            client.emit(event, data, trigger_update)
                         else:
                             if client.type == client_type:
-                                client.emit(event, data)
+                                client.emit(event, data, trigger_update)
         elif client_type is not None:
             for client in self.__clients.values():
                 if client.type == client_type:
-                    client.emit(event, data)
+                    client.emit(event, data, trigger_update)
         else:
             raise ValueError("'client_id', 'client_type', or 'room' must be provided")
 
@@ -541,7 +550,7 @@ class IoTManager(object):
             return
 
         if not kwargs.pop("called_by_client", False):
-            self.__clients[client].join(room, called_by_manager=True)
+            self.__clients[client].leave(room, called_by_manager=True)
 
         if self.__rooms.get(room, None) is None:
             return
@@ -623,7 +632,8 @@ class IoTManager(object):
         pass
 
 
-def emit(event: str, data: sendable, client_id: str = None, client_type: str = None, room: str = None):
+def emit(event: str, data: sendable, client_id: str = None, client_type: str = None, room: str = None,
+         trigger_update: bool = True):
     """
     Emit function for sending data to a single client, group of client types, or room of clients.
 
@@ -636,12 +646,18 @@ def emit(event: str, data: sendable, client_id: str = None, client_type: str = N
                         specifier.
     :param room: A room id of which all clients should be specified. If client_type is provided it will act as a
                  filter and only send the data ot clients of that type which are also in the specified room.
+    :param trigger_update: If this emit should trigger the managers on_update handler. This is useful for preventing
+                           unnecessary triggers of the on_update function if for example it is used to update a database
+                           for data persistence.
     :return:
     """
+    if client_id is None and client_type is None and room is None:
+        client_id = flask.request.g["client"].id
+
     return flask.current_app.extensions["iot.io"].emit(event, data, client_id, client_type, room)
 
 
-def join(client: Union[IoTClient, str], room: str):
+def join(room: str, client: Union[IoTClient, str] = None):
     """
     Add a client to a specified room.
 
@@ -649,10 +665,13 @@ def join(client: Union[IoTClient, str], room: str):
     :param room: ID of the room to be joined as a string
     :return:
     """
-    return flask.current_app.extensions["iot.io"].emit(client, room)
+    if client is None:
+        client = flask.request.g["client"].id
+
+    return flask.current_app.extensions["iot.io"].join(client, room)
 
 
-def leave(client: Union[IoTClient, str], room: str):
+def leave(room: str, client: Union[IoTClient, str] = None):
     """
     Remove a client from a specified room.
 
@@ -660,7 +679,10 @@ def leave(client: Union[IoTClient, str], room: str):
     :param room: ID of the room to be left as a string
     :return:
     """
-    return flask.current_app.extensions["iot.io"].emit(client, room)
+    if client is None:
+        client = flask.request.g["client"].id
+
+    return flask.current_app.extensions["iot.io"].leave(client, room)
 
 
 def close_room(room: str):
