@@ -503,27 +503,45 @@ class TestStringEndpointValidator(TestCase):
 
 
 class TestEnumEndpointValidator(TestCase):
-    validator = EnumEndpointValidator("abc", "def", "ghi", {
-        "a": "Apple",
-        "b": "Banana",
-        "c": "Cucumber"
+    validator_1 = EnumEndpointValidator("abc", "def", "ghi", {
+        "Apple": "a",
+        "Banana": "b",
+        "Cucumber": "c"
+    })
+
+    validator_2 = EnumEndpointValidator("abc", "def", "ghi", {
+        "Apple": 1,
+        "Banana": 2,
+        "Cucumber": 3
     })
 
     def test_validate(self):
-        self.assertEqual(self.validator.validate("a"), ValidationResponse.VALID)
-        self.assertEqual(self.validator.validate("b"), ValidationResponse.VALID)
-        self.assertEqual(self.validator.validate("c"), ValidationResponse.VALID)
-        self.assertEqual(self.validator.validate("d"), ValidationResponse.INVALID_ENUM_VALUE)
-        self.assertEqual(self.validator.validate("abc"), ValidationResponse.INVALID_ENUM_VALUE)
-        self.assertEqual(self.validator.validate(1), ValidationResponse.INVALID_TYPE)
+        self.assertEqual(self.validator_1.validate("a"), ValidationResponse.VALID)
+        self.assertEqual(self.validator_1.validate("b"), ValidationResponse.VALID)
+        self.assertEqual(self.validator_1.validate("c"), ValidationResponse.VALID)
+        self.assertEqual(self.validator_1.validate("d"), ValidationResponse.INVALID_ENUM_VALUE)
+        self.assertEqual(self.validator_1.validate("abc"), ValidationResponse.INVALID_ENUM_VALUE)
+        self.assertEqual(self.validator_1.validate(1.1), ValidationResponse.INVALID_TYPE)
+        self.assertEqual(self.validator_2.validate(1), ValidationResponse.VALID)
+        self.assertEqual(self.validator_2.validate(2), ValidationResponse.VALID)
+        self.assertEqual(self.validator_2.validate(3), ValidationResponse.VALID)
+        self.assertEqual(self.validator_2.validate(4), ValidationResponse.INVALID_ENUM_VALUE)
+        self.assertEqual(self.validator_2.validate(123), ValidationResponse.INVALID_ENUM_VALUE)
 
     def test_parse(self):
-        val = self.validator.parse(self.validator.__dict__())
+        val_1 = self.validator_1.parse(self.validator_1.__dict__())
 
-        self.assertEqual(val.id, self.validator.id)
-        self.assertEqual(val.name, self.validator.name)
-        self.assertEqual(val.special_id, self.validator.special_id)
-        self.assertEqual(val.values, self.validator.values)
+        self.assertEqual(val_1.id, self.validator_1.id)
+        self.assertEqual(val_1.name, self.validator_1.name)
+        self.assertEqual(val_1.special_id, self.validator_1.special_id)
+        self.assertEqual(val_1.values, self.validator_1.values)
+
+        val_2 = self.validator_2.parse(self.validator_2.__dict__())
+
+        self.assertEqual(val_2.id, self.validator_2.id)
+        self.assertEqual(val_2.name, self.validator_2.name)
+        self.assertEqual(val_2.special_id, self.validator_2.special_id)
+        self.assertEqual(val_2.values, self.validator_2.values)
 
         self.assertEqual(EnumEndpointValidator.parse({
             "id": "a",
@@ -545,11 +563,11 @@ class TestEnumEndpointValidator(TestCase):
             "constraints": {
                 "values": {
                     "a": "Apple",
-                    "b": 5,
+                    "b": {},
                     "c": "Cucumber"
                 }
             }
-        }), EndpointParseResponse.CONSTRAINT_VALUES_VALUES_MUST_BE_STRINGS)
+        }), EndpointParseResponse.CONSTRAINT_VALUES_VALUES_MUST_BE_INT_OR_STRING)
 
         self.assertEqual(EnumEndpointValidator.parse({
             "id": "a",
@@ -574,11 +592,20 @@ class TestEnumEndpointValidator(TestCase):
         }), EndpointParseResponse.CONSTRAINT_VALUES_MUST_BE_DICT)
 
     def test__dict__(self):
-        data = self.validator.__dict__()
+        data_1 = self.validator_1.__dict__()
 
-        self.assertEqual(data.get("id", None), self.validator.id)
-        self.assertEqual(data.get("name", None), self.validator.name)
-        self.assertEqual(data.get("type", None), EndpointType.ENUM.value)
-        self.assertEqual(data.get("specialId", None), self.validator.special_id)
-        self.assertIsNotNone(data.get("constraints", None))
-        self.assertEqual(data.get("constraints").get("values", None), self.validator.values)
+        self.assertEqual(data_1.get("id", None), self.validator_1.id)
+        self.assertEqual(data_1.get("name", None), self.validator_1.name)
+        self.assertEqual(data_1.get("type", None), EndpointType.ENUM.value)
+        self.assertEqual(data_1.get("specialId", None), self.validator_1.special_id)
+        self.assertIsNotNone(data_1.get("constraints", None))
+        self.assertEqual(data_1.get("constraints").get("values", None), self.validator_1.values)
+
+        data_2 = self.validator_2.__dict__()
+
+        self.assertEqual(data_2.get("id", None), self.validator_2.id)
+        self.assertEqual(data_2.get("name", None), self.validator_2.name)
+        self.assertEqual(data_2.get("type", None), EndpointType.ENUM.value)
+        self.assertEqual(data_2.get("specialId", None), self.validator_2.special_id)
+        self.assertIsNotNone(data_2.get("constraints", None))
+        self.assertEqual(data_2.get("constraints").get("values", None), self.validator_2.values)
